"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const mixed_1 = require("../helpers/mixed");
const download_1 = require("./download");
async function postinstall() {
    const result = await download_1.default();
    switch (result.tag) {
        case "Exit":
            return result.statusCode;
        case "Success":
            return linkTools(result.tools);
    }
}
exports.default = postinstall;
function linkTools(tools) {
    console.error(mixed_1.bold("Links:"));
    const nodeModulesPath = mixed_1.findClosest("node_modules");
    if (nodeModulesPath === undefined) {
        console.error("No node_modules/ folder found. Install your npm dependencies before running this script.");
        return 1;
    }
    const nodeModulesBinPath = path.join(nodeModulesPath, ".bin");
    try {
        fs.mkdirSync(nodeModulesBinPath, { recursive: true });
    }
    catch (errorAny) {
        const error = errorAny;
        console.error(`Failed to create ${nodeModulesBinPath}: ${error.message}`);
    }
    for (const tool of tools) {
        const linkPath = path.join(nodeModulesBinPath, tool.name);
        try {
            const target = fs.readlinkSync(linkPath);
            if (target === tool.absolutePath) {
                console.error(`${mixed_1.bold(`${tool.name} ${tool.version}`)} link already exists: ${mixed_1.dim(`${linkPath} -> ${target}`)}`);
                continue;
            }
            else {
                console.error(wrongSymlinkError(linkPath, target, tool.absolutePath));
                return 1;
            }
        }
        catch (errorAny) {
            const error = errorAny;
            switch (error.code) {
                case "ENOENT":
                    // Does not exist yet – move on.
                    break;
                case "EINVAL":
                    console.error(`${linkPath} already exists, but is not a link to ${tool.absolutePath}\n${error.message}\nRemove it and try again.`);
                    return 1;
                default:
                    console.error(`Failed to create ${linkPath}:\n${error.message}`);
                    return 1;
            }
        }
        try {
            fs.symlinkSync(tool.absolutePath, linkPath);
        }
        catch (errorAny) {
            const error = errorAny;
            console.error(`Failed to create ${linkPath}: ${error.message}`);
            return 1;
        }
        console.error(`${mixed_1.bold(`${tool.name} ${tool.version}`)} link created: ${mixed_1.dim(`${linkPath} -> ${tool.absolutePath}`)}`);
    }
    return 0;
}
function wrongSymlinkError(linkPath, actual, expected) {
    return `
${linkPath} already exists, but links to something else.
Expected: ${expected}
Actual:   ${actual}
Remove it and try again.
`.trim();
}
