"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const mixed_1 = require("../helpers/mixed");
const parse_1 = require("../helpers/parse");
function validate() {
    const parseResult = parse_1.findReadAndParseElmToolingJson();
    switch (parseResult.tag) {
        case "ElmToolingJsonNotFound":
            console.error(parseResult.message);
            return 1;
        case "ReadAsJsonObjectError":
            console.error(mixed_1.bold(parseResult.elmToolingJsonPath));
            console.error(parseResult.message);
            return 1;
        case "Parsed": {
            const elmJsonExists = parse_1.validateFileExists(path.join(path.dirname(parseResult.elmToolingJsonPath), "elm.json"));
            const elmJsonErrors = elmJsonExists.tag === "Exists"
                ? []
                : [
                    {
                        path: ["elm.json"],
                        message: `There should be an elm.json next to elm-tooling.json\n${elmJsonExists.message}`,
                    },
                ];
            const entrypointsErrors = parseResult.entrypoints === undefined
                ? []
                : getEntrypointsErrors(parseResult.entrypoints);
            const toolsErrors = parseResult.tools === undefined
                ? { tag: "Error", errors: [] }
                : getToolsErrors(parseResult.tools);
            const validationErrors = [
                ...elmJsonErrors,
                ...parseResult.unknownFields.map((field) => ({
                    path: [field],
                    message: `Unknown field`,
                })),
                ...entrypointsErrors,
                ...toolsErrors.errors,
            ];
            console.error(mixed_1.bold(parseResult.elmToolingJsonPath));
            if (validationErrors.length === 0) {
                console.error("No errors found.");
                return 0;
            }
            else {
                console.error("");
                console.error(parse_1.printFieldErrors(validationErrors));
                if (toolsErrors.tag === "Missing" && toolsErrors.errors.length > 0) {
                    console.error("");
                    console.error(`${mixed_1.dim("To download missing tools:")}\n    elm-tooling download`);
                }
                console.error("");
                console.error(mixed_1.elmToolingJsonDocumentationLink);
                return 1;
            }
        }
    }
}
exports.default = validate;
function getEntrypointsErrors(fieldResult) {
    switch (fieldResult.tag) {
        case "Error":
            return fieldResult.errors;
        case "Parsed":
            return [];
    }
}
function getToolsErrors(fieldResult) {
    switch (fieldResult.tag) {
        case "Error":
            return { tag: "Error", errors: fieldResult.errors };
        case "Parsed":
            return {
                tag: "Missing",
                errors: fieldResult.parsed.missing.map((tool) => ({
                    path: ["tools", tool.name],
                    message: `File does not exist: ${tool.absolutePath}`,
                })),
            };
    }
}
