"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const download_1 = require("./commands/download");
const init_1 = require("./commands/init");
const postinstall_1 = require("./commands/postinstall");
const validate_1 = require("./commands/validate");
const mixed_1 = require("./helpers/mixed");
const parse_1 = require("./helpers/parse");
const help = `
${mixed_1.bold("elm-tooling init")}
    Create a sample elm-tooling.json in the current directory

${mixed_1.bold("elm-tooling validate")}
    Validate the closest elm-tooling.json

${mixed_1.bold("elm-tooling download")}
    Download the tools in the closest elm-tooling.json to:
    ${mixed_1.dim(parse_1.elmToolingInstallPath)}

${mixed_1.bold("elm-tooling postinstall")}
    Download the tools in the closest elm-tooling.json
    and create links to them in node_modules/.bin/

${mixed_1.bold("Environment variables:")}
    ${mixed_1.bold("ELM_HOME")}
        Customize where tools will be downloaded.
        The Elm compiler uses this variable too for where to store packages.

    ${mixed_1.bold("NO_COLOR")}
        Disable colored output.

${mixed_1.bold("Documentation:")}
    https://github.com/lydell/elm-tooling.json/tree/master/cli
`.trim();
async function run(argv) {
    // So far no command takes any further arguments.
    // Let each command handle this when needed.
    if (argv.length > 1) {
        console.error(`Expected no extra arguments but got: ${argv.slice(1).join(" ")}`);
        return 1;
    }
    switch (argv[0]) {
        case undefined:
        case "-h":
        case "-help":
        case "--help":
        case "help":
            console.log(help);
            return 0;
        case "init":
            return await init_1.default();
        case "validate":
            return validate_1.default();
        case "download": {
            const result = await download_1.default();
            switch (result.tag) {
                case "Exit":
                    return result.statusCode;
                case "Success":
                    return 0;
            }
        }
        case "postinstall":
            return await postinstall_1.default();
        default:
            console.error(`Unknown command: ${argv[0]}`);
            return 1;
    }
}
run(process.argv.slice(2)).then((exitCode) => {
    process.exit(exitCode);
}, (error) => {
    console.error("Unexpected error", error);
    process.exit(1);
});
